export declare type Album = Timestamps & Image_2 & {
    id: number;
    title: string;
    normalized_title: string;
    release: Date;
    review_comment: string | null;
    edition: Date | null;
    edition_description: string | null;
    album_artists: AlbumArtist[];
    album_labels: AlbumLabel[];
};

export declare type AlbumArtist = AlbumArtistParams & {
    normalized_name: string;
    separator: string | null;
};

export declare type AlbumArtistParams = {
    artist_id: number;
    name: string;
    order: number;
    separator?: string;
};

export declare type AlbumLabel = {
    label_id: number;
    catalogue_number: string | null;
};

export declare class AlbumModule extends CRUDModule<AlbumParams, Album> {
    constructor(baseURL: string);
    index(apiToken: ApiToken, scope?: AlbumsScope): AsyncGenerator<Album[], Album[], void>;
    destroyEmpty(apiToken: ApiToken): Promise<boolean>;
    merge(apiToken: ApiToken, id: number | string, sourceId: number | string): Promise<Album>;
}

export declare interface AlbumParams {
    album: {
        title: string;
        release: Date;
        review_comment?: string;
        edition?: Date;
        edition_description?: string;
        image?: ImageParams;
        album_artists: AlbumArtistParams[];
        album_labels: AlbumLabel[];
    };
}

export declare class AlbumsScope extends Scope {
    artist(id: number | string): this;
    filter(string: string): this;
    label(id: number | string): this;
    labels(ids: number[] | string[]): this;
}

export declare type ApiToken = string;

export declare type Artist = Timestamps & Image_2 & {
    id: number;
    name: string;
    normalized_name: string;
    review_comment: string | null;
};

export declare class ArtistModule extends CRUDModule<ArtistParams, Artist> {
    constructor(baseURL: string);
    index(apiToken: ApiToken, scope?: ArtistsScope): AsyncGenerator<Artist[], Artist[], void>;
    destroyEmpty(apiToken: ApiToken): Promise<boolean>;
    merge(apiToken: ApiToken, id: number | string, sourceId: number | string): Promise<Artist>;
}

export declare interface ArtistParams {
    artist: {
        name: string;
        review_comment?: string;
        image?: ImageParams;
    };
}

export declare class ArtistsScope extends Scope {
    filter(string: string): this;
}

export declare type AuthToken = {
    id: number;
    user_id: number;
    user_agent: string;
    application?: string;
};

export declare class AuthTokenModule extends BaseModule {
    constructor(baseURL: string);
    index(apiToken: ApiToken): AsyncGenerator<AuthToken[], AuthToken[], void>;
    create(object: AuthTokenParams): Promise<AuthTokenWithToken>;
    read(apiToken: ApiToken, id: number, retryOptions?: RetryOptions): Promise<AuthToken>;
    destroy(apiToken: ApiToken, id: number | string): Promise<boolean>;
}

export declare interface AuthTokenParams {
    name: string;
    password: string;
    auth_token?: {
        user_agent?: string;
        application?: string;
    };
}

export declare type AuthTokenWithToken = AuthToken & {
    token: ApiToken;
};

declare class BaseModule {
    path: string;
    baseURL: string;
    constructor(baseURL: string, path: string);
    get url(): string;
}

export declare type Codec = {
    id: number;
    mimetype: string;
    extension: string;
};

export declare type CodecConversion = {
    id: number;
    name: string;
    ffmpeg_params: string;
    resulting_codec_id: number;
};

export declare class CodecConversionModule extends CRUDModule<CodecConversionParams, CodecConversion> {
    constructor(baseURL: string);
}

export declare interface CodecConversionParams {
    codec_conversion: {
        name: string;
        ffmpeg_params: string;
        resulting_codec_id: number;
    };
}

export declare class CodecModule extends CRUDModule<CodecParams, Codec> {
    constructor(baseURL: string);
}

export declare interface CodecParams {
    codec: {
        mimetype: string;
        extension: string;
    };
}

export declare type CoverFilename = {
    id: number;
    filename: string;
};

export declare class CoverFilenameModule extends CRUDModule<CoverFilenameParams, CoverFilename> {
    constructor(baseURL: string);
}

export declare interface CoverFilenameParams {
    cover_filename: {
        filename: string;
    };
}

declare class CRDModule<Params, ReturnType> extends BaseModule {
    index(apiToken: ApiToken): AsyncGenerator<ReturnType[], ReturnType[], void>;
    create(apiToken: ApiToken, object: Params): Promise<ReturnType>;
    read(apiToken: ApiToken, id: number, retryOptions?: RetryOptions): Promise<ReturnType>;
    destroy(apiToken: ApiToken, id: number | string): Promise<boolean>;
}

export declare function createApiClient(baseURL: string): {
    albums: AlbumModule;
    artists: ArtistModule;
    auth_tokens: AuthTokenModule;
    codec_conversions: CodecConversionModule;
    codecs: CodecModule;
    cover_filenames: CoverFilenameModule;
    genres: GenreModule;
    image_types: ImageTypeModule;
    labels: LabelModule;
    locations: LocationModule;
    playlists: PlaylistModule;
    plays: PlayModule;
    rescans: RescanModule;
    tracks: TrackModule;
    users: UserModule;
};

declare class CRUDModule<Params, ReturnType> extends CRDModule<Params, ReturnType> {
    update(apiToken: ApiToken, id: number, object: Params): Promise<ReturnType>;
}

export declare type Genre = {
    id: number;
    name: string;
    normalized_name: string;
};

export declare class GenreModule extends CRUDModule<GenreParams, Genre> {
    constructor(baseURL: string);
    destroyEmpty(apiToken: ApiToken): Promise<boolean>;
    merge(apiToken: ApiToken, id: number, sourceId: number): Promise<Genre>;
}

export declare interface GenreParams {
    genre: {
        name: string;
    };
}

declare type Image_2 = {
    image: string | null;
    image100: string | null;
    image250: string | null;
    image500: string | null;
    image_type: string | null;
};
export { Image_2 as Image }

export declare type ImageParams = {
    data: string | null;
    filename?: string;
    mimetype?: string;
};

export declare type ImageType = {
    id: number;
    mimetype: string;
    extension: string;
};

export declare class ImageTypeModule extends CRUDModule<ImageTypeParams, ImageType> {
    constructor(baseURL: string);
}

export declare interface ImageTypeParams {
    image_type: {
        mimetype: string;
        extension: string;
    };
}

export declare type Label = {
    id: number;
    name: string;
    normalized_name: string;
};

export declare class LabelModule extends CRUDModule<LabelParams, Label> {
    constructor(baseURL: string);
    destroyEmpty(apiToken: ApiToken): Promise<boolean>;
    merge(apiToken: ApiToken, id: number, sourceId: number): Promise<Label>;
}

export declare interface LabelParams {
    label: {
        name: string;
    };
}

declare type Location_2 = {
    id: number;
    path: string;
};
export { Location_2 as Location }

export declare class LocationModule extends CRDModule<LocationParams, Location_2> {
    constructor(baseURL: string);
}

export declare interface LocationParams {
    location: {
        path: string;
    };
}

export declare type Play = {
    id: number;
    played_at: Date;
    track_id: number;
    user_id: number;
};

export declare type Playlist = Timestamps & {
    name: string;
    description: string | null;
    user_id: number;
    access: PlaylistAccess;
    playlist_type: PlaylistType;
    item_ids: number[];
};

export declare const enum PlaylistAccess {
    SHARED = "shared",
    PERSONAL = "personal",
    SECRET = "secret"
}

export declare interface PlaylistItemParams {
    playlist: {
        item_id: number;
        item_type: "Album" | "Artist" | "Track";
    };
}

export declare class PlaylistModule extends CRUDModule<PlaylistParams, Playlist> {
    constructor(baseURL: string);
    addItem(apiToken: ApiToken, id: number, object: PlaylistItemParams): Promise<null>;
}

export declare interface PlaylistParams {
    playlist: {
        name: string;
        description?: string;
        access?: PlaylistAccess;
        playlist_type: PlaylistType;
        item_ids: [];
    };
}

export declare const enum PlaylistType {
    ALBUM = "album",
    ARTIST = "artist",
    TRACK = "track"
}

export declare class PlayModule extends BaseModule {
    constructor(baseURL: string);
    index(apiToken: ApiToken, scope?: PlaysScope): AsyncGenerator<Play[], Play[], void>;
    create(apiToken: ApiToken, object: PlayParams): Promise<Play>;
    stats(apiToken: ApiToken, scope?: PlaysScope): AsyncGenerator<PlayStat[], PlayStat[], void>;
}

export declare interface PlayParams {
    play: {
        track_id: number;
        played_at: Date;
    };
}

export declare class PlaysScope extends Scope {
    album(id: number | string): this;
    artist(id: number | string): this;
    playedBefore(date: Date): this;
    playedAfter(date: Date): this;
}

export declare type PlayStat = {
    last_played_at: Date;
    track_id: number;
    user_id: number;
    count: number;
    total_length: number;
};

declare type RequestDelayFunction = (attempt: number, error: Error | null, response: Response | null) => number;

declare type RequestRetryOnFunction = (attempt: number, error: Error | null, response: Response | null) => boolean | Promise<boolean>;

export declare type Rescan = {
    error_text: string | null;
    warning_text: string | null;
    processed: number;
    running: boolean;
    finished_at: Date;
    location_id: number;
};

export declare class RescanModule extends BaseModule {
    constructor(baseURL: string);
    index(apiToken: ApiToken): AsyncGenerator<Rescan[], Rescan[], void>;
    startAll(apiToken: ApiToken): Promise<Rescan>;
    start(apiToken: ApiToken, id: number): Promise<Rescan>;
    show(apiToken: ApiToken, id: number, retryOptions?: RetryOptions): Promise<Rescan>;
}

export declare interface RetryOptions {
    retries?: number;
    retryDelay?: number | RequestDelayFunction;
    retryOn?: number[] | RequestRetryOnFunction;
}

export declare class Scope {
    scopes: {
        key: string;
        query: string | number;
    }[];
    get finalQuery(): string;
    addScope(key: string, query: number | string): this;
    addScopesFromArray(key: string, queries: number[] | string[]): this;
}

export declare type Timestamps = {
    created_at: Date;
    updated_at: Date;
};

export declare type Track = Timestamps & {
    id: number;
    title: string;
    normalized_title: string;
    number: number;
    album_id: number;
    review_comment: string | null;
    genre_ids: number[];
    codec_id: number | null;
    length: number | null;
    bitrate: number | null;
    location_id: number | null;
    audio_file_id: number | null;
    track_artists: TrackArtist[];
    filename: string | null;
    sample_rate: number | null;
    bit_depth: number | null;
};

export declare type TrackArtist = TrackArtistParams & {
    normalized_name: string;
};

export declare type TrackArtistParams = {
    artist_id: number;
    name: string;
    role: TrackArtistRole;
    order: number;
    hidden: boolean;
};

export declare enum TrackArtistRole {
    main = "main",
    performer = "performer",
    composer = "composer",
    conductor = "conductor",
    remixer = "remixer",
    producer = "producer",
    arranger = "arranger"
}

export declare class TrackModule extends CRUDModule<TrackParams, Track> {
    constructor(baseURL: string);
    index(apiToken: ApiToken, scope?: TracksScope): AsyncGenerator<Track[], Track[], void>;
    destroyEmpty(apiToken: ApiToken): Promise<boolean>;
    merge(apiToken: ApiToken, id: number, sourceId: number): Promise<Track>;
}

export declare interface TrackParams {
    track: {
        title: string;
        number: number;
        album_id: number;
        review_comment?: string;
        genre_ids: number[];
        track_artists: TrackArtistParams[];
    };
}

export declare class TracksScope extends Scope {
    album(id: number | string): this;
    artist(id: number | string): this;
    filter(string: string): this;
    genre(id: number | string): this;
}

export declare type User = {
    id: number;
    name: string;
    permission: UserPermission;
};

export declare class UserModule extends CRUDModule<UserParams, User> {
    constructor(baseURL: string);
}

export declare interface UserParams {
    user: {
        name: string;
        password: string;
        password_confirmation: string;
        permission?: UserPermission;
    };
}

export declare enum UserPermission {
    user = "user",
    moderator = "moderator",
    admin = "admin"
}

export { }
